#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import urllib


from django.utils.deprecation import MiddlewareMixin
from opentracing import Format
from opentracing.ext import tags
from .conf import *
from .request_context import get_current_span, span_in_context, span_out_context

logger = logging.getLogger(__name__)


class TraceMiddleware(MiddlewareMixin):
    """"use jaeger_client realizing tracing"""

    def __init__(self, get_response=None):
        self.get_response = get_response
        self._tracer = None
        self._http_config = get_tracing_config().get("http_requests", {})
        self._tracer_config = get_tracer_config()

        # Initialize tracer if HTTP tracing is enabled
        if is_component_enabled("http_requests"):
            from initial_tracer import initialize_global_tracer
            self._tracer = initialize_global_tracer()

    def _should_ignore_request(self, request) -> bool:
        """
        Check if the request should be ignored based on configuration.
        :param: request:
        :return: True if request should be ignored, False otherwise.
        """
        if not is_component_enabled("http_requests"):
            return True

        ignore_urls = self._http_config.get("ignore_urls", [])
        request_path = request.path_info

        return any(
            ignore_url in request_path
            for ignore_url in ignore_urls
        )

    @staticmethod
    def _parse_wsgi_headers(request):
        """
        HTTP headers are presented in WSGI environment with 'HTTP_' prefix.
        This method finds those headers, removes the prefix, converts
        underscores to dashes, and converts to lower case.
        :param request:
        :return: returns a dictionary of headers
        """
        prefix = 'HTTP_'
        p_len = len(prefix)
        # use .items() despite suspected memory pressure bc GC occasionally
        # collects wsgi_environ.iteritems() during iteration.
        headers = {
            key[p_len:].replace('_', '-').lower():
                val for (key, val) in request.environ.items()
            if key.startswith(prefix)}
        setattr(request, 'headers', headers)

    @staticmethod
    def full_url(request):
        """
        Build the full URL from WSGI environ variables.
        Taken from:
        http://legacy.python.org/dev/peps/pep-3333/#url-reconstruction
        :return: Reconstructed URL from WSGI environment.
        """
        environ = request.environ
        url = environ['wsgi.url_scheme'] + '://'

        if environ.get('HTTP_HOST'):
            url += environ['HTTP_HOST']
        else:
            url += environ['SERVER_NAME']

            if environ['wsgi.url_scheme'] == 'https':
                if environ['SERVER_PORT'] != '443':
                    url += ':' + environ['SERVER_PORT']
            else:
                if environ['SERVER_PORT'] != '80':
                    url += ':' + environ['SERVER_PORT']

        url += urllib.parse.quote(environ.get('SCRIPT_NAME', ''))
        url += urllib.parse.quote(environ.get('PATH_INFO', ''))
        if environ.get('QUERY_STRING'):
            url += '?' + environ['QUERY_STRING']
        setattr(request, 'full_url', url)

    def process_request(self, request):
        """
        Process incoming HTTP request and start tracing span.
        :param request:
        :return:
        """
        if self._should_ignore_request(request):
            return

        # Parse headers and build URL
        self._parse_wsgi_headers(request)
        self.full_url(request)

        # Extract parent span context from request headers
        try:
            parent_ctx = self._tracer.extract(
                Format.HTTP_HEADERS,
                carrier=request.headers
            )
        except Exception as e:
            logger.exception(f'Failed to extract parent context:{e}')
            parent_ctx = None

        operation_name = '{} {}'.format(request.method, request.path)

        # Create standard tags for the request span
        span_tags = {
            tags.SPAN_KIND: tags.SPAN_KIND_RPC_SERVER,
            tags.HTTP_URL: request.full_url,
            tags.HTTP_METHOD: request.method,
            tags.COMPONENT: get_service_name() or "django"
        }

        remote_ip = request.environ.get('REMOTE_ADDR')
        if remote_ip:
            span_tags[tags.PEER_HOST_IPV4] = remote_ip

        remote_port = request.environ.get('REMOTE_PORT')
        if remote_port:
            span_tags[tags.PEER_PORT] = remote_port

        user_agent = request.META.get("HTTP_USER_AGENT")
        if user_agent:
            max_length = self._http_config.get("max_tag_value_length", 1024)
            span_tags["http.user_agent"] = user_agent[:max_length]

        span = self._tracer.start_span(
            operation_name=operation_name,
            child_of=parent_ctx,
            tags=span_tags)

        # # Add tracing headers to request
        if self._http_config.get("trace_headers", True):
            carrier = {}
            try:
                self._tracer.inject(
                    span_context=span.context,
                    format=Format.HTTP_HEADERS,
                    carrier=carrier
                )
                for key, value in carrier.items():
                    request.headers[key] = value
            except Exception as e:
                logger.debug(f"Failed to inject tracing headers: {e}")

        # Store span in context
        span_in_context(span)

    def process_response(self, request, response):
        span = get_current_span()

        # Add trace-id header to response
        trace_id_header = self._tracer_config.get("trace_id_header", "trace-id")
        response[trace_id_header] = request.headers.get(trace_id_header, "")

        if not span:
            # logger.exception('Can not get valid span for tracing.')
            return response

        try:
            span.set_tag(tags.HTTP_STATUS_CODE, response.status_code)
        except Exception as e:
            logger.exception(f'Error setting response tags for tracing:{e}')
        finally:
            span.finish()
            span_out_context()

        return response



"""
Database instrumentation for Django ORM queries.
Automatically traces database operations with query details and performance metrics.
"""
import logging
import time

from django.db.backends.utils import CursorWrapper
from opentracing.ext import tags

from ..conf import is_component_enabled, get_tracing_config
from ..initial_tracer import initialize_global_tracer
from ..request_context import get_current_span

logger = logging.getLogger(__name__)


class TracingCursorWrapper(CursorWrapper):
    """Database cursor wrapper that adds tracing to SQL queries."""

    def __init__(self, cursor, db):
        super().__init__(cursor, db)
        self._tracer = initialize_global_tracer()
        self._config = get_tracing_config().get("database", {})

    def _should_ignore_tracing(self, sql: str) -> bool:
        """Check if the query should be ignored based on configuration."""
        if not is_component_enabled("database"):
            return True

        ignore_sqls = self._config.get("ignore_sqls", [])

        return any(
            ignore_sql in sql.upper()
            for ignore_sql in ignore_sqls
        )

    def _create_span(self, sql: str, params=None):
        """Create tracing span for database query."""
        parent_span = get_current_span()
        operation_name = "DB_QUERY"

        span = self._tracer.start_span(
            operation_name=operation_name,
            child_of=parent_span
        )

        # Set standard database tags
        span.set_tag(tags.SPAN_KIND, tags.SPAN_KIND_RPC_CLIENT)
        span.set_tag(tags.COMPONENT, "django.db")
        span.set_tag(tags.DATABASE_TYPE, self.db.vendor)
        span.set_tag(tags.DATABASE_INSTANCE, self.db.settings_dict.get("NAME", ""))
        span.set_tag(tags.DATABASE_USER, self.db.settings_dict.get("USER", ""))
        span.set_tag(tags.PEER_HOST_IPV4, self.db.settings_dict.get("HOST", ""))
        span.set_tag(tags.PEER_PORT, self.db.settings_dict.get("PORT", ""))

        # Add SQL statement (optionally truncated)
        if self._config.get("log_sql", False):
            max_length = self._config.get("max_query_length", 1000)
            span.set_tag(tags.DATABASE_STATEMENT, sql[:max_length])

        return span

    def execute(self, sql, params=None):
        """Execute SQL with tracing."""
        if not self._should_ignore(sql):
            return super().execute(sql, params)

        span = self._create_span(sql, params)
        start_time = time.time()

        try:
            result = super().execute(sql, params)

            # Check for slow queries
            duration_ms = (time.time() - start_time) * 1000
            span.set_tag("db.duration_ms", round(duration_ms, 2))

            # Check for slow queries
            slow_threshold = self._config.get("slow_query_threshold", 100)
            if duration_ms > slow_threshold:
                span.set_tag("db.slow_query", True)
                span.log_kv({
                    "event": "slow_query",
                    "duration_ms": duration_ms,
                    "threshold_ms": slow_threshold,
                })

            return result

        except Exception as e:
            span.set_tag(tags.ERROR, True)
            span.log_kv({
                "event": "error",
                "error.kind": e.__class__.__name__,
                "error.object": str(e),
                "message": str(e),
            })
            raise
        finally:
            span.finish()

    def executemany(self, sql, param_list):
        """Execute many SQL statements with tracing."""
        if self._should_ignore_tracing(sql):
            return super().execute(sql, param_list)

        span = self._create_span(sql, param_list)
        start_time = time.time()

        try:
            result = super().executemany(sql, param_list)

            # Calculate query duration
            duration_ms = (time.time() - start_time) * 1000
            span.set_tag("db.duration_ms", round(duration_ms, 2))

            return result

        except Exception as e:
            span.set_tag(tags.ERROR, True)
            span.log_kv({
                "event": "error",
                "error.kind": e.__class__.__name__,
                "error.object": str(e),
                "message": str(e),
            })
            raise
        finally:
            span.finish()


class DatabaseInstrumentation:
    """Database instrumentation manager."""

    @classmethod
    def install(cls):
        """Install database instrumentation."""
        if not is_component_enabled("database"):
            return

        from django.db.backends.base.base import BaseDatabaseWrapper

        def traced_make_cursor(self, cursor):
            """Create traced cursor wrapper."""
            return TracingCursorWrapper(cursor, self)

        # Monkey patch the make_cursor method
        BaseDatabaseWrapper.make_cursor = traced_make_cursor
        logger.info("Database instrumentation installed")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import requests

from .conf import is_component_enabled
from .instrumentation.http import TracingHTTPAdapter

logger = logging.getLogger(__name__)


class HttpClient:
    """
    HTTP client with built-in tracing support.
    Drop-in replacement for requests with automatic tracing.
    """

    def __init__(self, base_url: str = None, headers: dict = None, timeout: int = 10, retry: int = 0):
        self.base_url = base_url
        self.default_headers = headers or {}
        self.timeout = timeout
        self.retry = retry
        self.session = requests.Session()

        # Install tracing adapter
        if is_component_enabled("http_requests"):
            adapter = TracingHTTPAdapter(max_retries=self.retry)
            self.session.mount('http://', adapter)
            self.session.mount('https://', adapter)

    def _prepare_url(self, url: str) -> str:
        """Prepare URL with base URL if needed."""
        if self.base_url and not url.startswith(('http://', 'https://')):
            return f"{self.base_url.rstrip('/')}/{url.lstrip('/')}"
        return url

    def _prepare_headers(self, headers: dict = None) -> dict:
        """Merge default headers with request headers."""
        merged_headers = self.default_headers.copy()
        if headers:
            merged_headers.update(headers)
        return merged_headers

    def get(self, url: str, params=None, headers=None, **kwargs):
        """Make GET request with tracing."""
        return self.session.get(
            self._prepare_url(url),
            params=params,
            headers=self._prepare_headers(headers),
            timeout=kwargs.get('timeout', self.timeout),
            **kwargs
        )

    def post(self, url: str, data=None, headers=None, **kwargs):
        """Make POST request with tracing."""
        return self.session.post(
            self._prepare_url(url),
            data=data,
            headers=self._prepare_headers(headers),
            timeout=kwargs.get('timeout', self.timeout),
            **kwargs
        )

    def put(self, url: str, data=None, headers=None, **kwargs):
        """Make PUT request with tracing."""
        return self.session.put(
            self._prepare_url(url),
            data=data,
            headers=self._prepare_headers(headers),
            timeout=kwargs.get('timeout', self.timeout),
            **kwargs
        )

    def patch(self, url: str, data=None, headers=None, **kwargs):
        """Make PATCH request with tracing."""
        return self.session.patch(
            self._prepare_url(url),
            data=data,
            headers=self._prepare_headers(headers),
            timeout=kwargs.get('timeout', self.timeout),
            **kwargs
        )

    def delete(self, url: str, headers=None, **kwargs):
        """Make DELETE request with tracing."""
        return self.session.delete(
            self._prepare_url(url),
            headers=self._prepare_headers(headers),
            timeout=kwargs.get('timeout', self.timeout),
            **kwargs
        )


http_client = HttpClient()



"""
Instrumentation package for different components.
Contains auto-instrumentation for various libraries and frameworks.
"""
from .http import HTTPInstrumentation
from .database import DatabaseInstrumentation
from .redis import RedisInstrumentation
from .celery import CeleryInstrumentation
from .rocketmq import RocketMQInstrumentation

__all__ = [
    "HTTPInstrumentation",
    "DatabaseInstrumentation",
    "RedisInstrumentation",
    "CeleryInstrumentation",
    "RocketMQInstrumentation",
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Tracer initialization and management.
Handles Jaeger tracer setup and global tracer access.
"""

import logging

import opentracing
from jaeger_client import Config

from .conf import get_tracer_config, get_service_name
from .exceptions import TracingInitializationError

logger = logging.getLogger(__name__)


def initialize_global_tracer():
    _config = Config(
        config=get_tracer_config(),
        service_name=get_service_name(),
        validate=True,
    )
    if _config.initialized():
        tracer = opentracing.tracer
    else:
        try:
            # use uwsgi
            # multi-processing, fork(2) issues see:
            # https://github.com/jaegertracing/jaeger-client-python/issues/60
            # https://github.com/jaegertracing/jaeger-client-python/issues/31
            from uwsgidecorators import postfork

            @postfork
            def post_fork_initialize_jaeger():
                _config.initialize_tracer()

            tracer = opentracing.tracer
        except ImportError:
            # use gunicorn etc.
            tracer = _config.initialize_tracer()
        finally:
            if not _config.initialized():
                raise TracingInitializationError(
                    'Failed to initialize Jaeger tracer.'
                )
            logger.info(f'Jaeger tracer initialized for service: {get_service_name()}')
    return tracer



"""
Redis instrumentation for Redis operations.
Automatically traces Redis commands with performance metrics.
"""
import logging
import time

from opentracing.ext import tags

from ..conf import is_component_enabled, get_tracing_config
from ..initial_tracer import initialize_global_tracer
from ..request_context import get_current_span

logger = logging.getLogger(__name__)


class TracingRedisConnection:
    """Redis connection wrapper that adds tracing."""

    def __init__(self, connection):
        self._connection = connection
        self._tracer = initialize_global_tracer()
        self._config = get_tracing_config().get("redis", {})

    def __getattr__(self, name):
        """Delegate to original connection for unknown attributes."""
        return getattr(self._connection, name)

    def _should_ignore_tracing(self, command_name: str) -> bool:
        """Determine if the Redis command should be traced."""
        if not is_component_enabled("redis"):
            return True

        ignore_commands = self._config.get("ignore_commands", [])

        return any(
            ignore_command.upper() in command_name.upper()
            for ignore_command in ignore_commands
        )


    def _create_span(self, command_name: str, args: tuple = None):
        """Create tracing span for Redis command."""
        parent_span = get_current_span()
        operation_name = "REDIS"

        span = self._tracer.start_span(
            operation_name=operation_name,
            child_of=parent_span
        )

        # Set standard Redis tags
        span.set_tag(tags.SPAN_KIND, tags.SPAN_KIND_RPC_CLIENT)
        span.set_tag(tags.COMPONENT, "redis")
        span.set_tag(tags.DATABASE_TYPE, "redis")

        # Add connection information
        connection_kwargs = getattr(self._connection, "connection_kwargs", {})
        if "host" in connection_kwargs:
            span.set_tag(tags.PEER_HOST_IPV4, connection_kwargs["host"])
        if "port" in connection_kwargs:
            span.set_tag(tags.PEER_PORT, connection_kwargs["port"])
        if "db" in connection_kwargs:
            span.set_tag("db.redis.database_index", connection_kwargs["db"])

        statement = command_name.upper()
        if args:
            statement += " " + str(args[0])

        # Add statement (optionally truncated)
        if self._config.get("log_command", False):
            max_length = self._config.get("max_command_length", 500)
            span.set_tag(tags.DATABASE_STATEMENT, statement[:max_length])

        return span

    def send_command(self, *args, **kwargs):
        """Execute Redis command with tracing."""
        if not args:
            return self._connection.send_command(*args, **kwargs)

        command_name = str(args[0])

        if self._should_ignore_tracing(command_name):
            return self._connection.send_command(*args, **kwargs)

        span = self._create_span(command_name, args[1:])
        start_time = time.time()

        try:
            result = self._connection.send_command(*args, **kwargs)

            # Calculate command duration
            duration_ms = (time.time() - start_time) * 1000
            span.set_tag("db.redis.duration_ms", round(duration_ms, 2))

            return result

        except Exception as e:
            span.set_tag(tags.ERROR, True)
            span.log_kv({
                "event": "error",
                "error.kind": e.__class__.__name__,
                "error.object": str(e),
                "message": str(e),
            })
            raise
        finally:
            span.finish()


class RedisInstrumentation:
    """Redis instrumentation manager."""

    @classmethod
    def install(cls):
        """Install Redis instrumentation."""
        if not is_component_enabled("redis"):
            return

        try:
            import redis
            from redis.connection import Connection

            class TracedConnection(Connection):
                """Redis connection with tracing support."""

                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self._tracing_wrapper = None

                def connect(self):
                    """Connect and wrap with tracing."""
                    result = super().connect()
                    if not self._tracing_wrapper:
                        self._tracing_wrapper = TracingRedisConnection(self)
                    return result

                def send_command(self, *args, **kwargs):
                    """Execute command with tracing."""
                    if self._tracing_wrapper:
                        return self._tracing_wrapper.send_command(*args, **kwargs)
                    return super().send_command(*args, **kwargs)

            # Replace Redis connection class
            redis.connection.Connection = TracedConnection
            redis.Connection = TracedConnection

            logger.info("Redis instrumentation installed")

        except ImportError:
            logger.warning("Redis package not found, skipping Redis instrumentation")



"""
Celery instrumentation for distributed task tracing.
Automatically traces Celery task execution and message passing.
"""
import logging
import time

from opentracing import Format
from opentracing.ext import tags

from ..conf import is_component_enabled, get_tracing_config
from ..initial_tracer import initialize_global_tracer
from ..request_context import get_current_span, span_in_context, span_out_context

logger = logging.getLogger(__name__)


class CeleryInstrumentation:
    """Celery instrumentation manager."""

    @classmethod
    def install(cls):
        """Install Celery instrumentation."""
        if not is_component_enabled("celery"):
            return

        try:
            from celery.app.task import Task

            # Monkey patch Task.apply_async for trace injection
            original_apply_async = Task.apply_async

            def traced_apply_async(self, args=None, kwargs=None, **options):
                """Apply async with tracing context injection."""
                return cls._inject_trace_context(self, original_apply_async, args, kwargs, **options)

            Task.apply_async = traced_apply_async

            logger.info("Celery instrumentation installed")

        except ImportError:
            logger.warning("Celery package not found, skipping Celery instrumentation")

    @classmethod
    def _should_ignore_tracing(cls, task_name: str) -> bool:
        """Determine if task should be traced."""
        if not is_component_enabled("celery"):
            return True

        config = get_tracing_config().get("celery", {})
        ignore_tasks = config.get("ignore_tasks", [])

        return task_name not in ignore_tasks

    @classmethod
    def _inject_trace_context(cls, task, original_method, args=None, kwargs=None, **options):
        """Inject tracing context into task headers."""
        if cls._should_ignore_tracing(task.name):
            return original_method(task, args, kwargs, **options)

        tracer = initialize_global_tracer()
        parent_span = get_current_span()
        operation_name = "CELERY"

        if parent_span:
            # Create span for task publishing
            span = tracer.start_span(
                operation_name=operation_name,
                child_of=parent_span
            )

            span.set_tag(tags.SPAN_KIND, tags.SPAN_KIND_PRODUCER)
            span.set_tag(tags.COMPONENT, "celery")
            span.set_tag("celery.task_name", task.name)
            span.set_tag("celery.task_id", options.get("task_id", ""))

            # Inject trace context into task headers
            headers = options.get("headers", {})
            carrier = {}

            try:
                tracer.inject(
                    span_context=span.context,
                    format=Format.TEXT_MAP,
                    carrier=carrier
                )

                # Add tracing headers to request
                for key, value in carrier.items():
                    headers[key] = value

            except Exception as e:
                logger.debug(f"Failed to inject trace context: {e}")

            try:
                return original_method(task, args, kwargs, **options)
            finally:
                span.finish()

        return original_method(task, args, kwargs, **options)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="django-jaeger-middleware-plus",
    version="0.1.2",
    author="zhaishuaishuai",
    author_email="zhaishuaishuai001@gmail.com",
    description="A Django middleware for distributed tracing with Jaeger",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lcoolcool/django-jaeger-middleware-plus",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
    ],
    python_requires=">=3.7",
    install_requires=[
        "Django>=2.2",
        "jaeger-client>=4.6.1",
        "opentracing>=2.4.0",
        "requests>=2.25.1",
    ],
    extras_require={
        "dev": [
            "pytest>=6.0",
            "pytest-django>=2.2",
            "black>=21.0",
            "flake8>=3.8",
            "mypy>=0.812",
        ],
    },
    keywords=["django", "django-jaeger-middleware", "jaeger" "opentracing" "microservice"],
    project_urls={
        "Bug Reports": "https://github.com/lcoolcool/django-jaeger-middleware-plus/issues",
        "Source": "https://github.com/lcoolcool/django-jaeger-middleware-plus",
        "Documentation": "https://github.com/lcoolcool/django-jaeger-middleware-plus",
    },
)



"""
RocketMQ instrumentation for message queue tracing.
Automatically traces RocketMQ message production and consumption.
"""
import logging

from opentracing import Format
from opentracing.ext import tags

from ..conf import is_component_enabled, get_tracing_config
from ..initial_tracer import initialize_global_tracer
from ..request_context import get_current_span, span_in_context

logger = logging.getLogger(__name__)


class RocketMQInstrumentation:
    """RocketMQ instrumentation manager."""

    @classmethod
    def install(cls):
        """Install RocketMQ instrumentation."""
        if not is_component_enabled("rocketmq"):
            return

        try:
            # Try to import RocketMQ client
            from rocketmq.client import Producer, PushConsumer

            # Monkey patch Producer
            cls._patch_producer(Producer)

            # Monkey patch PushConsumer
            cls._patch_consumer(PushConsumer)

            logger.info("RocketMQ instrumentation installed")

        except ImportError:
            logger.warning("RocketMQ package not found, skipping RocketMQ instrumentation")

    @classmethod
    def _should_ignore_tracing(cls, topic: str) -> bool:
        """Determine if topic should be traced."""
        if not is_component_enabled("rocketmq"):
            return True

        config = get_tracing_config().get("rocketmq", {})
        ignore_topics = config.get("ignore_topics", [])

        return topic not in ignore_topics

    @classmethod
    def _patch_producer(cls, producer_class):
        """Patch RocketMQ Producer class."""
        original_send_sync = producer_class.send_sync
        original_send_oneway = producer_class.send_oneway

        def traced_send_sync(self, msg, timeout=3000):
            """Send message synchronously with tracing."""
            return cls._trace_send_message(self, original_send_sync, msg, "send_sync", timeout=timeout)

        def traced_send_oneway(self, msg):
            """Send message oneway with tracing."""
            return cls._trace_send_message(self, original_send_oneway, msg, "send_oneway")

        # Replace methods
        producer_class.send_sync = traced_send_sync
        producer_class.send_oneway = traced_send_oneway

    @classmethod
    def _patch_consumer(cls, consumer_class):
        """Patch RocketMQ Consumer class."""
        original_subscribe = consumer_class.subscribe

        def traced_subscribe(self, topic, callback, expression="*"):
            """Subscribe to topic with tracing wrapper."""
            def traced_callback(msg):
                return cls._trace_consume_message(msg, callback)

            return original_subscribe(self, topic, traced_callback, expression)

        consumer_class.subscribe = traced_subscribe

    @classmethod
    def _trace_send_message(cls, producer, original_method, msg, operation, **kwargs):
        """Trace message sending operations."""
        topic = msg.topic

        if cls._should_ignore_tracing(topic):
            return original_method(producer, msg, **kwargs)

        tracer = initialize_global_tracer()
        config = get_tracing_config().get("rocketmq", {})
        current_span = get_current_span()

        # Create span for message production
        span = tracer.start_span(
            operation_name=f"ROCKETMQ",
            child_of=current_span
        )

        span.set_tag(tags.SPAN_KIND, tags.SPAN_KIND_PRODUCER)
        span.set_tag(tags.COMPONENT, "rocketmq")
        span.set_tag(tags.MESSAGE_BUS_DESTINATION, topic)
        span.set_tag("rocketmq.operation", operation)
        span.set_tag("rocketmq.topic", topic)

        # Add message tags
        if hasattr(msg, "tags") and msg.tags:
            span.set_tag("rocketmq.tags", msg.tags)

        if hasattr(msg, "keys") and msg.keys:
            span.set_tag("rocketmq.keys", msg.keys)

        # Add message body
        if hasattr(msg, "body") and msg.body and config.get("trace_message_body", False):
            body_content = msg.body
            span.set_tag("rocketmq.body", str(body_content))

        # Inject tracing context into message properties
        try:
            carrier = {}
            tracer.inject(
                span_context=span.context,
                format=Format.TEXT_MAP,
                carrier=carrier
            )

            # Add trace context to message properties
            if not hasattr(msg, "properties"):
                msg.properties = {}
            if msg.properties is None:
                msg.properties = {}

            msg.properties.update(carrier)

        except Exception as e:
            logger.debug(f"Failed to inject trace context into RocketMQ message: {e}")

        try:
            result = original_method(producer, msg, **kwargs)

            # For synchronous operations, finish span immediately
            if operation == "send_sync":
                if hasattr(result, "status"):
                    if result.status == 0:  # Success
                        span.set_tag("rocketmq.send_status", "success")
                        if hasattr(result, "msg_id"):
                            span.set_tag("rocketmq.msg_id", result.msg_id)
                    else:
                        span.set_tag("rocketmq.send_status", "failed")
                        span.set_tag(tags.ERROR, True)
                span.finish()
            elif operation == "send_oneway":
                span.set_tag("rocketmq.send_status", "oneway")
                span.finish()

            return result

        except Exception as e:
            span.set_tag(tags.ERROR, True)
            span.log_kv({
                "event": "error",
                "error.kind": e.__class__.__name__,
                "error.object": str(e),
                "message": str(e),
            })
            span.finish()
            raise

    @classmethod
    def _trace_consume_message(cls, msg, original_callback):
        """Trace message consumption."""
        topic = msg.topic

        if cls._should_ignore_tracing(topic):
            return original_callback(msg)

        tracer = initialize_global_tracer()
        config = get_tracing_config().get("rocketmq", {})

        # Extract parent context from message properties
        parent_context = None
        if hasattr(msg, "properties") and msg.properties:
            try:
                parent_context = tracer.extract(
                    format=Format.TEXT_MAP,
                    carrier=msg.properties
                )
            except Exception as e:
                logger.debug(f"Failed to extract trace context from RocketMQ message: {e}")

        # Create span for message consumption
        span = tracer.start_span(
            operation_name=f"ROCKETMQ",
            child_of=parent_context
        )

        span.set_tag(tags.SPAN_KIND, tags.SPAN_KIND_CONSUMER)
        span.set_tag(tags.COMPONENT, "rocketmq")
        span.set_tag(tags.MESSAGE_BUS_DESTINATION, topic)
        span.set_tag("rocketmq.topic", topic)

        # Add message information
        if hasattr(msg, "msg_id"):
            span.set_tag("rocketmq.msg_id", msg.msg_id)

        if hasattr(msg, "tags") and msg.tags:
            span.set_tag("rocketmq.tags", msg.tags)

        if hasattr(msg, "keys") and msg.keys:
            span.set_tag("rocketmq.keys", msg.keys)

        if hasattr(msg, "queue_id"):
            span.set_tag("rocketmq.queue_id", msg.queue_id)

        if hasattr(msg, "born_timestamp"):
            span.set_tag("rocketmq.born_timestamp", msg.born_timestamp)

        # Add message body
        if hasattr(msg, "body") and msg.body and config.get("trace_message_body", False):
            body_content = msg.body
            span.set_tag("rocketmq.body", str(body_content))

        try:
            with span_in_context(span):
                result = original_callback(msg)

            span.set_tag("rocketmq.consume_status", "success")

            return result

        except Exception as e:
            span.set_tag(tags.ERROR, True)
            span.set_tag("rocketmq.consume_status", "failed")
            span.log_kv({
                "event": "error",
                "error.kind": e.__class__.__name__,
                "error.object": str(e),
                "message": str(e),
            })
            raise
        finally:
            span.finish()
